﻿using Infragistics.Controls.Charts;
using Infragistics.Windows;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace DataChartAxisLabels
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        CategoryDateTimeXAxis xAxis;

        public MainWindow()
        {
            InitializeComponent();

            xAxis = new CategoryDateTimeXAxis();
            xAxis.DateTimeMemberPath = "Date";
            xAxis.Label = this.FindResource("labelTemplate");

            theChart.Axes.Add(xAxis);
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            theChart.Axes.Clear();
            theChart.Series.Clear();

            TestData data = new TestData();

            NumericYAxis yAxis = new NumericYAxis();

            LineSeries series = new LineSeries();
            series.ItemsSource = data;
            series.XAxis = xAxis;
            series.YAxis = yAxis;
            series.ValueMemberPath = "Value";

            xAxis.ItemsSource = data;

            theChart.Axes.Add(xAxis);
            theChart.Axes.Add(yAxis);
            theChart.Series.Add(series);
        }     
    }

    public class TestData : ObservableCollection<TestDataItem>
    {
        private static Random _rand = new Random();

        public TestData()
        {
            DateTime now = new DateTime(2011, 3, 1);
            DateTime dt = new DateTime(2011, 3, 1);
            double curr = 10.0;
            while (dt < now.AddYears(2))
            {
                if (_rand.NextDouble() < .5)
                {
                    curr += _rand.NextDouble() * 2.0;
                }
                else
                {
                    curr -= _rand.NextDouble() * 2.0;
                }

                Add(new TestDataItem()
                {
                    Date = dt,
                    Value = curr
                });

                dt = dt.AddHours(.5);
            }
        }
    }

    public class TestDataItem
    {
        public DateTime Date { get; set; }
        public double Value { get; set; }
    }

    public class DateLabel : ContentControl
    {
        private StackPanel _stack = new StackPanel();
        private TextBlock _firstLine = new TextBlock();
        private TextBlock _secondLine = new TextBlock();

        #region Chart

        public XamDataChart Chart
        {
            get { return (XamDataChart)GetValue(ChartProperty); }
            set { SetValue(ChartProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Chart.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ChartProperty =
            DependencyProperty.Register("Chart", typeof(XamDataChart), typeof(DateLabel), new PropertyMetadata(null, OnChartChanged));

        private static void OnChartChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            //if (e.OldValue != null)
            //    (d as DateLabel).CleanupEvents();
            //else
            //    (d as DateLabel).SetupEvents();
        }

        #endregion

        #region Chart events


        //private void SetupEvents()
        //{
        //    Chart.RefreshCompleted += Chart_RefreshCompleted;
        //}

        //private void CleanupEvents()
        //{
        //    Chart.RefreshCompleted -= Chart_RefreshCompleted;
        //}

        //void Chart_RefreshCompleted(object sender, EventArgs e)
        //{
        //    Update();
        //}

        #endregion

        public DateLabel()
        {
            this.Loaded += DateLabel_Loaded;
            _firstLine.HorizontalAlignment = HorizontalAlignment.Center;
            _secondLine.HorizontalAlignment = HorizontalAlignment.Center;
            _stack.Children.Add(_firstLine);
            _stack.Children.Add(_secondLine);

            Content = _stack;
        }
        
        void DateLabel_Loaded(object sender, RoutedEventArgs e)
        {
            DateLabel DL = sender as DateLabel;
            AxisItemDataContext DC = DL.DataContext as AxisItemDataContext;
            DateTime x = (DateTime)(DC.Item);
            this.Date = x;
            Update();
        }

        public static readonly DependencyProperty DateProperty =
            DependencyProperty.Register("Date",
            typeof(DateTime),
            typeof(DateLabel),
            new PropertyMetadata(DateTime.MinValue,
                (o, e) => (o as DateLabel)
                    .OnDateChanged(e.OldValue, e.NewValue)));

        private void OnDateChanged(object oldValue, object newValue)
        {
            this.Date = (DateTime)newValue;
        }

        public DateTime Date
        {
            get { return (DateTime)GetValue(DateProperty); }
            set { SetValue(DateProperty, value); }
        }

        public void Update()
        {
            // make sure all the data labels are present.
            if (!AreLabelsValid()) return;

            _firstLine.Text = "";
            _secondLine.Text = "";

            var scale = DetermineAxisScale();
            if (scale == AxisScale.Unknown)
            {
                return;
            }

            var displayDifferentiator = ShouldDisplayDifferentiator(scale);
            var firstListFormat = GetFirstLineFormat(scale);
            var secondLineFormat = GetSecondLineFormat(scale);

            var firstText = Date.ToString(firstListFormat);
            var secondText = Date.ToString(secondLineFormat);

            _firstLine.Text = firstText;
            if (displayDifferentiator &&
                scale != AxisScale.Years &&
                !string.IsNullOrEmpty(secondLineFormat))
            {
                _secondLine.Text = secondText;
            }
        }

        bool AreLabelsValid()
        {
            // make sure all labels exist first.
            if (GetDateLabels().Any(d => d == null))
                return false;

            return true;
        }

        private string GetSecondLineFormat(AxisScale scale)
        {
            switch (scale)
            {
                case AxisScale.Years:
                    return "";
                    break;
                case AxisScale.Months:
                    return "yyyy";
                    break;
                case AxisScale.Days:
                    if (ShouldDisplayDifferentiator(AxisScale.Months))
                    {
                        return "MM/yyyy";
                    }
                    else
                    {
                        return "MM";
                    }
                    break;
                case AxisScale.Hours:
                    if (ShouldDisplayDifferentiator(AxisScale.Months))
                    {
                        return "MM/dd/yyyy";
                    }
                    else if (ShouldDisplayDifferentiator(AxisScale.Days))
                    {
                        return "MM/dd";
                    }
                    else if (ShouldDisplayDifferentiator(AxisScale.Hours))
                    {
                        return "dd";
                    }
                    else
                    {
                        return "";
                    }
                    break;
                case AxisScale.Minutes:
                    if (ShouldDisplayDifferentiator(AxisScale.Months))
                    {
                        return "MM/dd/yyyy";
                    }
                    else if (ShouldDisplayDifferentiator(AxisScale.Days))
                    {
                        return "MM/dd";
                    }
                    else if (ShouldDisplayDifferentiator(AxisScale.Hours))
                    {
                        return "dd";
                    }
                    else
                    {
                        return "";
                    }
                    break;
            }

            return "";
        }

        private string GetFirstLineFormat(AxisScale scale)
        {
            switch (scale)
            {
                case AxisScale.Years:
                    return "yyyy";
                    break;
                case AxisScale.Months:
                    return "MMM";
                    break;
                case AxisScale.Days:
                    return "dd";
                    break;
                case AxisScale.Hours:
                    return "hh:mm";
                    break;
                case AxisScale.Minutes:
                    return "hh:mm:ss";
                    break;
            }

            return "";
        }

        private bool ShouldDisplayDifferentiator(AxisScale scale)
        {
            var sorted = GetSortedLabels();
            Func<DateTime, int> getValue = (d) => 0;
            switch (scale)
            {
                case AxisScale.Years:
                    getValue = (d) => 0;
                    break;
                case AxisScale.Months:
                    getValue = (d) => d.Year;
                    break;
                case AxisScale.Days:
                    getValue = (d) => d.Month;
                    break;
                case AxisScale.Hours:
                    getValue = (d) => d.Day;
                    break;
                case AxisScale.Minutes:
                    getValue = (d) => d.Hour;
                    break;
            }

            int last = -1;
            foreach (var item in sorted)
            {
                var val = getValue(item.Date);
                if (last != val)
                {
                    last = val;
                    if (item == this)
                    {
                        return true;
                    }
                }

                if (item == this)
                {
                    break;
                }
            }

            return false;
        }

        private IEnumerable<DateLabel> GetSortedLabels()
        {
            var dateLabels = GetDateLabels();

            var sorted = from item in dateLabels
                         orderby item.Date
                         select item;

            return sorted;
        }

        private IEnumerable<DateLabel> GetDateLabels()
        {
            var axisPanel = Utilities.GetDescendantFromType(this.Chart, typeof(HorizontalAxisLabelPanel), false) as HorizontalAxisLabelPanel;
            var dateLabels = from item in axisPanel.Children.OfType<ContentControl>()
                             select (DateLabel)Utilities.GetDescendantFromType(item, typeof(DateLabel), false);
            
            return dateLabels;
        }

        private AxisScale DetermineAxisScale()
        {
            var sorted = GetSortedLabels();

            DateTime firstDate = sorted.First().Date;
            DateTime lastDate = sorted.Last().Date;


            if (!HasRepeats(AxisScale.Years, sorted))
            {
                return AxisScale.Years;
            }
            if (!HasRepeats(AxisScale.Months, sorted))
            {
                return AxisScale.Months;
            }
            if (!HasRepeats(AxisScale.Days, sorted))
            {
                return AxisScale.Days;
            }
            if (!HasRepeats(AxisScale.Hours, sorted))
            {
                return AxisScale.Hours;
            }

            return AxisScale.Minutes;
        }

        private bool HasRepeats(AxisScale scale, IEnumerable<DateLabel> sorted)
        {
            Func<DateTime, int> getValue = (d) => 0;
            switch (scale)
            {
                case AxisScale.Years:
                    getValue = (d) => d.Year;
                    break;
                case AxisScale.Months:
                    getValue = (d) => d.Month;
                    break;
                case AxisScale.Days:
                    getValue = (d) => d.Day;
                    break;
                case AxisScale.Hours:
                    getValue = (d) => d.Hour;
                    break;
                case AxisScale.Minutes:
                    getValue = (d) => d.Minute;
                    break;
            }

            int last = -1;
            foreach (var item in sorted)
            {
                var val = getValue(item.Date);
                if (last == val)
                {
                    return true;
                }

                if (last != val)
                {
                    last = val;
                }
            }

            return false;
        }      

        private enum AxisScale
        {
            Years,
            Months,
            Days,
            Hours,
            Minutes,
            Unknown
        }
    }  
}
